"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var React = require("react");
var _ = require("underscore");
var formCore_1 = require("../formCore");
var index_1 = require("../../../index");
var validationWrapper_1 = require("../validationWrapper");
var CodeInput = /** @class */ (function (_super) {
    tslib_1.__extends(CodeInput, _super);
    function CodeInput(props) {
        var _this = _super.call(this, props) || this;
        _this.uniq = Math.random();
        _this.state = { focusIndex: null };
        return _this;
    }
    CodeInput.prototype.focusNext = function (e) {
        var movingBack = false;
        var current = e.target;
        var el;
        var currentVal = current.value.trim();
        currentVal = currentVal.slice(0, this.props.lengthPerBox[this.state.focusIndex]);
        if (e.keyCode === 8) {
            movingBack = true;
            if (currentVal.length === 0) {
                el = current.previousSibling;
            }
            else {
                el = current;
            }
        }
        else {
            el = current.nextSibling;
        }
        if (currentVal.length < this.props.lengthPerBox[this.state.focusIndex] && !movingBack) {
            return;
        }
        if (el) {
            el.focus();
        }
    };
    CodeInput.prototype.buildValue = function () {
        var value = "";
        var current = document.getElementById("input_" + this.uniq + "_0");
        this.props.lengthPerBox.forEach(function (lpb, i) {
            value += current.value;
            current = current.nextSibling;
        });
        if (this.props.numeric) {
            value = parseInt(value);
        }
        if (this.props.onChange) {
            this.props.onChange(value);
        }
    };
    CodeInput.prototype.handlePaste = function (e) {
        var length = _.reduce(this.props.lengthPerBox, function (memo, num) {
            return memo + num;
        }, 0);
        var pasted = e.clipboardData.getData("text/plain").replace(/\s/g, "");
        pasted = pasted.substr(0, length);
        if (this.props.numeric) {
            var parsed = parseInt(pasted);
            if (isNaN(parsed)) {
                e.preventDefault();
                return false;
            }
        }
        var current = e.target;
        var splitArray = [];
        var currentIndex = 0;
        this.props.lengthPerBox.forEach(function (lpb, i) {
            var chunk = pasted.substr(currentIndex, lpb);
            currentIndex += lpb;
            splitArray.push(chunk);
            current.value = chunk;
            current = current.nextSibling;
        });
        this.buildValue();
    };
    CodeInput.prototype.handleFocus = function (index, input) {
        this.setState({ focusIndex: index }, function () {
            input.select();
            // ios safari fix
            input.setSelectionRange(0, 9999);
        });
    };
    CodeInput.prototype.componentDidMount = function () {
        var _this = this;
        if (this.props.value) {
            var currentIndex_1 = 0;
            this.props.lengthPerBox.forEach(function (lpb, i) {
                var input = document.getElementById("input_" + _this.uniq + "_" + i);
                var chunk = _this.props.value.substr(currentIndex_1, lpb);
                currentIndex_1 += lpb;
                input.value = chunk;
            });
        }
    };
    CodeInput.prototype.render = function () {
        var _this = this;
        var validationMessage = formCore_1.DataValidationMessage.get(this.props);
        var _a = this.props, onChange = _a.onChange, validationMode = _a.validationMode, attrs = tslib_1.__rest(_a, ["onChange", "validationMode"]);
        var classes = index_1.ClassHelpers.classNames("armstrong-input", "code-input", this.props.className, {
            "show-validation": validationMode !== "none" && validationMessage
        });
        return (React.createElement("div", { className: classes },
            React.createElement("div", null, this.props.lengthPerBox.map(function (lpb, i) { return (React.createElement("input", { id: "input_" + _this.uniq + "_" + i, readOnly: _this.props.readonly, className: "code-input-field", key: "cb_" + _this.uniq + "_" + i, type: _this.props.type || "text", placeholder: _this.props.placeholder, maxLength: lpb, onFocus: function (e) { return _this.handleFocus(i, e.target); }, onKeyUp: function (e) { return _this.focusNext(e); }, onPaste: function (e) { return _this.handlePaste(e); }, onChange: function (e) { return _this.buildValue(); } })); })),
            React.createElement(validationWrapper_1.ValidationLabel, { message: validationMessage, mode: validationMode })));
    };
    CodeInput.defaultProps = {
        lengthPerBox: [2, 2, 2],
        validationMode: "none"
    };
    return CodeInput;
}(React.Component));
exports.CodeInput = CodeInput;
